@@ -2,6 +2,7 @@ import os
 import json
 import gzip
 import time
+import hmac
 import asyncio
 import hashlib
 
@@ -20,9 +21,9 @@ class ApiProvider:
     """Provides HTTP-api requests and websocket requests."""
     def __init__(
             self, *, api_key='', api_secret='', from_env=False,
-            auth_required=True, timeout=3, retries=5,
-            root_url='https://api.crypto.com/v1/',
-            ws_root_url='wss://ws.crypto.com/kline-api/ws'):
+            auth_required=True, timeout=3, retries=20,
+            root_url='https://api.crypto.com/v2/',
+            ws_root_url='wss://stream.crypto.com/v2/'):
         self.api_key = api_key
         self.api_secret = api_secret
         self.root_url = root_url
@@ -30,8 +31,8 @@ class ApiProvider:
         self.timeout = timeout
         self.retries = retries
 
-        # NOTE: do not change this, due to crypto.com rate-limit 10-per second
-        self.semaphore = asyncio.Semaphore(10)
+        # NOTE: do not change this, due to crypto.com rate-limits
+        self.semaphore = asyncio.Semaphore(50)
 
         if not auth_required:
             return
@@ -50,59 +51,51 @@ class ApiProvider:
         if not self.api_secret:
             raise ValueError('Provide CRYPTOCOM_API_SECRET env value')
 
-    def _sign(self, data):
+    def _sign(self, path, data):
         data = data or {}
+        data['method'] = path
+
         sign_time = int(time.time() * 1000)
-        data.update({'time': sign_time, 'api_key': self.api_key})
-        sign = ''.join(f'{key}{data[key]}' for key in sorted(data))
-        sign = f'{sign}{self.api_secret}'
-        data['sign'] = hashlib.sha256(sign.encode('utf-8')).hexdigest()
+        data.update({'nonce': sign_time, 'api_key': self.api_key, 'id': 5959})
+
+        data_params = data.get('params', {})
+        if not data_params:
+            data['params'] = {}
+        params = ''.join(
+            f'{key}{data_params[key]}'
+            for key in sorted(data_params)
+        )
+
+        payload = f"{path}{data['id']}" \
+            f"{self.api_key}{params}{data['nonce']}"
+
+        data['sig'] = hmac.new(
+            self.api_secret.encode('utf-8'),
+            payload.encode('utf-8'),
+            hashlib.sha256
+        ).hexdigest()
         return data
 
-    async def ws_listen(self, data, timeout: int = None):
-        timeout = aiohttp.ClientTimeout(self.timeout)
-        count = 0
-
-        while count != self.retries:
-            async with aiohttp.ClientSession(timeout=timeout) as session:
-                try:
-                    async with session.ws_connect(self.ws_root_url) as ws:
-                        await ws.send_str(json.dumps(data))
-                        async for msg in ws:
-                            data = gzip.decompress(msg.data).decode('utf-8')
-                            data = json.loads(data)
-                            yield data
-                            count = 0
-                except asyncio.TimeoutError:
-                    if count == self.retries:
-                        raise ApiError(
-                            f"Timeout error, retries: {self.retries}")
-
-                    await asyncio.sleep(0.2)
-                    count += 1
-                    continue
-
-    async def ws_request(self, data, timeout: int = None):
-        async for data in self.ws_listen(data, timeout=timeout):
-            return data
-
     async def request(self, method, path, params=None, data=None, sign=False):
         if sign:
-            data = self._sign(data)
+            data = self._sign(path, data)
 
         timeout = aiohttp.ClientTimeout(total=self.timeout)
-
         for count in range(self.retries + 1):
             try:
                 async with aiohttp.ClientSession(timeout=timeout) as session:
                     async with self.semaphore:
                         resp = await session.request(
                             method, urljoin(self.root_url, path),
-                            params=params, data=data
+                            params=params, json=data,
+                            headers={'content-type': 'application/json'}
                         )
                         resp_json = await resp.json()
+                        if resp.status != 200:
+                            raise ApiError(
+                                f"Error: {resp_json}. Status: {resp.status}")
             except aiohttp.ClientConnectorError:
-                raise ApiError("Cannot connect to host {self.root_url}")
+                raise ApiError(f"Cannot connect to host {self.root_url}")
             except asyncio.TimeoutError:
                 if count == self.retries:
                     raise ApiError(
@@ -110,19 +103,20 @@ class ApiProvider:
                         f"Code: {resp.status}. Data: {data}"
                     )
 
-                await asyncio.sleep(1)
+                await asyncio.sleep(0.1)
                 continue
             except ContentTypeError:
                 if resp.status == 429:
-                    await asyncio.sleep(1)
+                    await asyncio.sleep(0.1)
                     continue
                 text = await resp.text()
                 raise ApiError(
                     f"Can't decode json, content: {text}. "
                     f"Code: {resp.status}")
 
-            if resp_json['code'] == '0':
-                return resp_json['data']
+            if resp_json['code'] == 0:
+                result = resp_json.get('result', {})
+                return result.get('data', {}) or result
 
             if count == self.retries:
                 raise ApiError(
@@ -131,7 +125,7 @@ class ApiProvider:
                     f"Data: {data}"
                 )
 
-            await asyncio.sleep(1)
+            await asyncio.sleep(0.1)
             continue
 
     async def get(self, path, params=None):
